@@ -49,7 +49,7 @@ use Imager::Search::Match ();
 
 use vars qw{$VERSION};
 BEGIN {
-	$VERSION = '0.02';
+	$VERSION = '0.04';
 }
 
 
@@ -109,16 +109,13 @@ sub new {
 	}
 
 	# Get the transforms
-	$self->{small_transform} ||= $self->small_transform;
-	unless ( _CODELIKE($self->{small_transform}) ) {
+	unless ( _CODELIKE($self->small_transform) ) {
 		Carp::croak("The small_transform param was not a CODE reference");
 	}
-	$self->{big_transform} ||= $self->big_transform;
-	unless ( _CODELIKE($self->{big_transform}) ) {
+	unless ( _CODELIKE($self->big_transform) ) {
 		Carp::croak("The big_transform param was not a CODE reference");
 	}
-	$self->{newline_transform} ||= $self->newline_transform;
-	unless ( _CODELIKE($self->{newline_transform}) ) {
+	unless ( _CODELIKE($self->newline_transform) ) {
 		Carp::croak("The newline_transform param was not a CODE reference");
 	}
 
@@ -159,32 +156,64 @@ sub small {
 
 =pod
 
+=head2 find
+
+The C<find> method compiles the search and target images in memory, and
+executes a single search, returning the position of the first match as a
+L<Imager::Search::Match> object.
+
+=cut
+
+sub find {
+	my $self  = shift;
+
+	# Load the strings.
+	# Do it by reference entirely for performance reasons.
+	# This avoids copying some potentially very large string.
+	my $small = '';
+	my $big   = '';
+	$self->_small_string( \$small );
+	$self->_big_string( \$big );
+
+	# Run the search
+	my @match = ();
+	my $bpp   = $self->bytes_per_pixel;
+	while ( scalar $big =~ /$small/gs ) {
+		my $p = $-[0];
+		push @match, Imager::Search::Match->from_position($self, $p / $bpp);
+		pos $big = $p + 1;
+	}
+	return @match;
+}
+
+=pod
+
 =head2 find_first
 
-The C<find_first> method is the only one implemented in this first release
-of L<Imager::Search>.
-
-It compiles the search and target images in memory, and executes a single
-search, returning the position of the first match as a
+The C<find_first> compiles the search and target images in memory, and
+executes a single search, returning the position of the first match as a
 L<Imager::Search::Match> object.
 
 =cut
 
 sub find_first {
 	my $self  = shift;
-	my $big   = ${$self->_big_string};
-	my $small = ${$self->_small_string};
-	my @match = ();
+
+	# Load the strings.
+	# Do it by reference entirely for performance reasons.
+	# This avoids copying some potentially very large string.
+	my $small = '';
+	my $big   = '';
+	$self->_small_string( \$small );
+	$self->_big_string( \$big );
+
+	# Run the search
+	my $bpp = $self->bytes_per_pixel;
 	while ( scalar $big =~ /$small/gs ) {
 		my $p = $-[0];
-		push @match, $p / 7;
-		pos $big = $p + 1;
-	}
-	if ( @match ) {
-		Imager::Search::Match->from_position($self, $match[0]);
-	} else {
-		return undef;
+		return Imager::Search::Match->from_position($self, $p / $bpp);
 	}
+	return undef;
 }
 
 
@@ -194,47 +223,20 @@ sub find_first {
 #####################################################################
 # Support Methods
 
-sub _big_string {
-	my $self   = shift;
-	my $height = $self->big->getheight;
-	my $string = '';
-	foreach my $row ( 0 .. $height - 1 ) {
-		$string .= $self->_big_scanline($row);
-	}
-	return \$string;
-}
-
-sub _big_scanline {
-	my $self = shift;
-	my $row  = shift;
-
-	# Get the colour array
-	my $col  = 0;
-	my $line = '';
-	my $func = $self->{big_transform};
-	foreach my $color ( $self->big->getscanline( y => $row ) ) {
-		my $string = &$func( $color );
-		unless ( _STRING($string) ) {
-			Carp::croak("Did not generate a search string for cell $row,$col");
-		}
-		$line .= $string;
-		$col++;
-	}
-
-	return $line;
-}
-
 sub _small_string {
-	my $self    = shift;
-	my $height  = $self->small->getheight;
-	my $pixels  = $self->big->getwidth - $self->small->getwidth;
-	my $func    = $self->{newline_transform};
-	my $newline = &$func( $pixels );
-	my $string  = $self->_small_scanline(0);
-	foreach my $row ( 1 .. $height - 1 ) {
-		$string .= $newline . $self->_small_scanline($row);
+	my $self        = shift;
+	my $scalar_ref  = shift;
+	my $height      = $self->small->getheight;
+	my $nl_width    = $self->big->getwidth - $self->small->getwidth;
+	my $nl_function = $self->newline_transform;
+	my $nl_string   = &$nl_function( $nl_width );
+	foreach my $row ( 0 .. $height - 1 ) {
+		$$scalar_ref .= $nl_string if $row;
+		$$scalar_ref .= $self->_small_scanline($row);
 	}
-	return \$string;
+
+	# Return the scalar reference as a convenience
+	return $scalar_ref;
 }
 
 sub _small_scanline {
@@ -244,7 +246,7 @@ sub _small_scanline {
 	# Get the colour array
 	my $col  = -1;
 	my $line = '';
-	my $func = $self->{small_transform};
+	my $func = $self->small_transform;
 	my $this = '';
 	my $more = 1;
 	foreach my $color ( $self->small->getscanline( y => $row ) ) {
@@ -266,6 +268,28 @@ sub _small_scanline {
 	return $line;
 }
 
+sub _big_string {
+	my $self       = shift;
+	my $scalar_ref = shift;
+	my $height     = $self->big->getheight;
+	my $func       = $self->big_transform;
+	foreach my $row ( 0 .. $height - 1 ) {
+		# Get the string for the row
+		my $col = 0;
+		foreach my $color ( $self->big->getscanline( y => $row ) ) {
+			my $pixel = &$func( $color );
+			unless ( _STRING($pixel) ) {
+				Carp::croak("Did not generate a search string for cell $row,$col");
+			}
+			$$scalar_ref .= $pixel;
+			$col++;
+		}
+	}
+
+	# Return the scalar reference as a convenience
+	return $scalar_ref;
+}
+
 1;
 
 =pod
