@@ -7,7 +7,7 @@ use base 'Imager::Search';
 
 use vars qw{$VERSION};
 BEGIN {
-	$VERSION = '0.02';
+	$VERSION = '0.04';
 }
 
 
@@ -44,6 +44,10 @@ sub __newline_transform {
 	return ".{$chars}";
 }
 
+sub bytes_per_pixel {
+	return 7;
+}
+
 1;
 
 __END__
